searchData={"items":[{"type":"module","title":"Md","doc":"`Md` is a markup parser allowing fully customized syntax definition and\nunderstanding the wide range of [markdown](https://www.markdownguide.org/) out of the box.\n\nIt is stream-aware, extendable, flexible, blazingly fast, with callbacks and more.","ref":"Md.html"},{"type":"module","title":"Main Focus - Md","doc":"This library is not yet another markdown parser, rather it’s a highly configurable\nand extendable parser for any custom markdown-like markup. It has been created\nmostly to allow custom markdown syntax, like `^foo^` for superscript, or `⇓bar⇓`\nfor subscript. It also supports [custom parsers](https://hexdocs.pm/md/Md.Parser.html)\nfor anything that cannot be handled with generic parsers, inspired by markdown\n(something more complex than standard markdown provides.)\n\nThe library provides callbacks for all the default syntax handlers, as well as for\ncustom handlers, allowing the on-fly modification of what’s currently being processed.\n\n`Md` parses the incoming stream **once** and keeps the state, producing an AST\nof the input document. It has an ability to recover from errors collecting them.\n\nIt currently does not support (and I frankly doubt it ever will)\nlists with embedded quotes, and other contrived syntax. If one needs to perfectly\nparse the common markdown, `Md` is probably not the correct choice.\n\nBut if one wants to easily extend syntax almost without limits, `Md` might be good.","ref":"Md.html#module-main-focus"},{"type":"module","title":"Markup Handling - Md","doc":"There are several different syntax patterns recognizable by `Md`. Those are:\n\n- `custom` — the custom parser implementing `Md.Parser` behavious would be called\n- `attributes` — inplace attributes for all tags `\"{{foo:'bar'}}\"` → `\" \"`\n- `substitute` — simple substitution, like `\" 2 `)\n- `brace` — a most common markdown feature, like text decoration or such (e. g. `**bold**`)","ref":"Md.html#module-markup-handling"},{"type":"module","title":"Syntax description - Md","doc":"The syntax must be configured at compile time (because `parse/2` handlers are\ngenerated in compile time.) It is a map, having `settings` key\n\n```elixir\nsettings: %{\n  outer: :p,\n  span: :span,\n  empty_tags: ~w|img hr br|a\n}\n```\n\nand `key ⇒ list_of_tuples` key-values, providing a text markup representation\nand its handling rules. Here is the excerpt from the default parser for `brace`s\n\n```elixir\n  brace: %{\n    \"*\" => %{tag: :b},\n    \"_\" => %{tag: :i},\n    \"**\" => %{tag: :strong, attributes: %{class: \"nota-bene\"}},\n    \"__\" => %{tag: :em},\n    \"~\" => %{tag: :s},\n    \"~~\" => %{tag: :del},\n    \"`\" => %{tag: :code, mode: :raw, attributes: %{class: \"code-inline\"}}\n  }\n```\n\nFor more examples of what properties are allowed for each kind of handlers,\nsee the sources (ATM.)","ref":"Md.html#module-syntax-description"},{"type":"module","title":"Predefined parsers - Md","doc":"`Md` comes with a generic predefined parser `Md.Parser.Default`, which includes\nall the markup currently supported by `Md`.\n\nCustom parser definition would be usually based on `Md.Parser.Syntax.Void` syntax\nas shown below\n\n```elixir\ndefmodule MyParser do\n  use Md.Parser\n\n  alias Md.Parser.Syntax.Void\n\n  @default_syntax Map.put(Void.syntax(), :settings, Void.settings())\n  @syntax @default_syntax |> Map.merge(%{\n    comment: [{\" \"}}],\n    paragraph: [\n      {\"##\", %{tag: :h2}},\n      {\"###\", %{tag: :h3}},\n      {\">\", %{tag: :blockquote}}\n    ],\n    list:\n      [\n        {\"- \", %{tag: :li, outer: :ul}},\n        {\"+ \", %{tag: :li, outer: :ol}}\n      ]\n    brace: [\n      {\"*\", %{tag: :b}},\n      {\"_\", %{tag: :i}},\n      {\"~\", %{tag: :s}},\n      {\"`\", %{tag: :code, mode: :raw, attributes: %{class: \"code-inline\"}}}\n    ]\n  })\nend\n```\n\n`@syntax` module attribute must be declared, or DSL used as shown below\n(declarations), or an argument in a call to `use Md.Parser`.\nThe separate declarations will be collected and merged.\n\n```elixir\ndefmodule MyDSLParser do\n  @my_syntax %{brace: [{\"***\", %{tag: \"u\"}}]}\n  \n  use Md.Parser, syntax: @my_syntax\n  import Md.Parser.DSL\n\n  comment \" \"}\n  ...\nend\n```\n\nInstead of `@syntax` module attribute, one might use\n\n- a parameter to `use Md.Parser` as `use Md.Parser, syntax: map()`\n- a DSL like `paragraph {\"#\", %{tag: :h1}}`.","ref":"Md.html#module-predefined-parsers"},{"type":"function","title":"Md.generate/1","doc":"","ref":"Md.html#generate/1"},{"type":"function","title":"Md.generate/2","doc":"","ref":"Md.html#generate/2"},{"type":"function","title":"Md.generate/3","doc":"","ref":"Md.html#generate/3"},{"type":"function","title":"Md.parse/2","doc":"Interface to the library. Use `parse/2` to parse the input to the state,\nuse `generate/{1,2}` to produce an _HTML_ out of the input.","ref":"Md.html#parse/2"},{"type":"function","title":"Examples - Md.parse/2","doc":"iex> Md.parse(\"   foo\")\n    %Md.Parser.State{ast: [{:p, nil, [\"foo\"]}], mode: [:finished]}\n\n    iex> Regex.replace(~r/\\s+/, Md.generate(\"*bold*\"), \"\")\n    \"  bold  \"\n\n    iex> Md.generate(\"It’s all *bold* and _italic_!\", Md.Parser.Default, format: :none)\n    \" It’s all  bold  and  italic ! \"","ref":"Md.html#parse/2-examples"},{"type":"module","title":"Md.Guards","doc":"Several guards for the proper UTF8 handling of input.","ref":"Md.Guards.html"},{"type":"module","title":"Examples - Md.Guards","doc":"iex> import Md.Guards\n    iex> with  >   with  >   with  >   with  >   with  >   with  >   with  >   with  >   with  > <- \"①\", do: is_utf8_digit(x)\n    true","ref":"Md.Guards.html#module-examples"},{"type":"macro","title":"Md.Guards.is_ascii_digit/1","doc":"","ref":"Md.Guards.html#is_ascii_digit/1"},{"type":"macro","title":"Md.Guards.is_ascii_punct/1","doc":"","ref":"Md.Guards.html#is_ascii_punct/1"},{"type":"macro","title":"Md.Guards.is_ascii_space/1","doc":"","ref":"Md.Guards.html#is_ascii_space/1"},{"type":"macro","title":"Md.Guards.is_non_ascii_digit/1","doc":"","ref":"Md.Guards.html#is_non_ascii_digit/1"},{"type":"macro","title":"Md.Guards.is_non_ascii_punct/1","doc":"","ref":"Md.Guards.html#is_non_ascii_punct/1"},{"type":"macro","title":"Md.Guards.is_non_ascii_space/1","doc":"","ref":"Md.Guards.html#is_non_ascii_space/1"},{"type":"macro","title":"Md.Guards.is_utf8_digit/1","doc":"","ref":"Md.Guards.html#is_utf8_digit/1"},{"type":"macro","title":"Md.Guards.is_utf8_punct/1","doc":"","ref":"Md.Guards.html#is_utf8_punct/1"},{"type":"macro","title":"Md.Guards.is_utf8_space/1","doc":"","ref":"Md.Guards.html#is_utf8_space/1"},{"type":"behaviour","title":"Md.Listener","doc":"The listener behaviour to attach to the parser to receive callbacks\nwhen the elements are encountered and processed.","ref":"Md.Listener.html"},{"type":"callback","title":"Md.Listener.element/2","doc":"","ref":"Md.Listener.html#c:element/2"},{"type":"type","title":"Md.Listener.attributes/0","doc":"","ref":"Md.Listener.html#t:attributes/0"},{"type":"type","title":"Md.Listener.branch/0","doc":"","ref":"Md.Listener.html#t:branch/0"},{"type":"type","title":"Md.Listener.callback/0","doc":"","ref":"Md.Listener.html#t:callback/0"},{"type":"type","title":"Md.Listener.context/0","doc":"","ref":"Md.Listener.html#t:context/0"},{"type":"type","title":"Md.Listener.element/0","doc":"","ref":"Md.Listener.html#t:element/0"},{"type":"type","title":"Md.Listener.leaf/0","doc":"","ref":"Md.Listener.html#t:leaf/0"},{"type":"type","title":"Md.Listener.parse_mode/0","doc":"","ref":"Md.Listener.html#t:parse_mode/0"},{"type":"type","title":"Md.Listener.state/0","doc":"","ref":"Md.Listener.html#t:state/0"},{"type":"type","title":"Md.Listener.trace/0","doc":"","ref":"Md.Listener.html#t:trace/0"},{"type":"behaviour","title":"Md.Parser","doc":"Interface to implement for the custopm parsers.\n\nCustom parsers might be used in syntax declaration when the generic functionality\nis not enough.\n\nLet’s consider one needs a specific handling of links with titles.\n\nThe generic engine does not support it, so one would need to implement a custom parser\nand instruct `Md.Parser` to use it with:\n\n```elixir\n# config/prod.exs\n\nconfig :md, syntax: %{\n  custom: [\n    {\"![\", {MyApp.Parsers.Img, %{}}},\n    ...\n  ]\n}\n```\n\nOnce the original parser would meet the `\"![\"` binary, it’d call `MyApp.Parsers.Img.parse/2`.\nThe latter must proceed until the tag is closed and return the remainder and the updated state\nas a tuple.","ref":"Md.Parser.html"},{"type":"function","title":"Md.Parser.generate/2","doc":"The main function transforming the MD input to XML output.\n\n`options` are passed to `XmlBuilder.generate/2` as is, save for\n  the special two special options, `parser:` and `walker:`.\n\n`parser:` option which is a module implementing `Parser` behaviour,\n  is telling what parser to use for parsing.\n\n`walker:` option which might be a function of arity 2,\n  or a tuple `{:pre | :post, fun}` where `fun` is a function of arity 1 or 2.\n\nIf passed, `XmlBuilder.{pre,post}walk/{2,3}` is being called before generation.\n\nIf `walker:` was not passed, or a `nil` value was returned from the accumulator,\n  this function returns the binary `XML` as is, otherwise it returns a tuple\n  `{XML, accumulator}` where `accumulator` is what has been returned from\n  underlying calls to `XmlBuilder.traverse/4`.","ref":"Md.Parser.html#generate/2"},{"type":"function","title":"Md.Parser.generate/3","doc":"","ref":"Md.Parser.html#generate/3"},{"type":"callback","title":"Md.Parser.parse/2","doc":"Takes a not parsed yet input and the state, returns the updated remainder and state.","ref":"Md.Parser.html#c:parse/2"},{"type":"type","title":"Md.Parser.parsing_stage/0","doc":"The type to be used in all the intermediate states of parsing.\n\nThe first element iof the tuple is the continuation (not parsed yet input,)\nand the latter is the current state.","ref":"Md.Parser.html#t:parsing_stage/0"},{"type":"module","title":"Md.Parser.Default","doc":"Default parser with all the features included.\n\nSupports a wide subset of markdown, including but not limited to:\n\n- bold/italic/strikeout/etc text decorations\n- comments\n- code blocks\n- tables\n- twitter handlers etc\n\nMight be a good start for those who just needs a fast markdown processing.","ref":"Md.Parser.Default.html"},{"type":"function","title":"Md.Parser.Default.listener/2","doc":"","ref":"Md.Parser.Default.html#listener/2"},{"type":"function","title":"Md.Parser.Default.syntax/0","doc":"","ref":"Md.Parser.Default.html#syntax/0"},{"type":"module","title":"Md.Parser.State","doc":"The internal state of the parser.","ref":"Md.Parser.State.html"},{"type":"type","title":"Md.Parser.State.t/0","doc":"","ref":"Md.Parser.State.html#t:t/0"},{"type":"behaviour","title":"Md.Parser.Syntax","doc":"The behaviour for the custom syntax suppliers","ref":"Md.Parser.Syntax.html"},{"type":"function","title":"Md.Parser.Syntax.merge/2","doc":"","ref":"Md.Parser.Syntax.html#merge/2"},{"type":"callback","title":"Md.Parser.Syntax.settings/0","doc":"The implementation should return settings for this particular syntax definition","ref":"Md.Parser.Syntax.html#c:settings/0"},{"type":"callback","title":"Md.Parser.Syntax.syntax/0","doc":"The implementation should return a syntax definition","ref":"Md.Parser.Syntax.html#c:syntax/0"},{"type":"type","title":"Md.Parser.Syntax.item/0","doc":"Syntax item definition","ref":"Md.Parser.Syntax.html#t:item/0"},{"type":"type","title":"Md.Parser.Syntax.settings/0","doc":"Settings for the parser","ref":"Md.Parser.Syntax.html#t:settings/0"},{"type":"type","title":"Md.Parser.Syntax.t/0","doc":"Syntax definition","ref":"Md.Parser.Syntax.html#t:t/0"},{"type":"module","title":"Md.Parser.Syntax.Void","doc":"Void syntax to be extended by custom implementations. Included for convenience.","ref":"Md.Parser.Syntax.Void.html"},{"type":"function","title":"Md.Parser.Syntax.Void.settings/0","doc":"Returns default values for outer tag, span tag and empty tags.","ref":"Md.Parser.Syntax.Void.html#settings/0"},{"type":"function","title":"Md.Parser.Syntax.Void.syntax/0","doc":"Empty syntax","ref":"Md.Parser.Syntax.Void.html#syntax/0"}],"content_type":"text/markdown"}